/*
 * bind: takes an url, a callback function and optionally allows setting the request
 *       method (GET or POST), message type ('xml' or 'formdata'), message (defaults to no message)
 *       and an optional error handler.
 *
 * As a special case, if type == 'formdata' and message is an object then the object's fields
 * are treated as key/value pairs and URLEncoded automatically(!).
 *
 * Note that lib/urlencode.js has to be included before using this class!
 */

function Ajax()
{
    this.bindObj = new XMLHttpRequest();
    this.bind = function(url,
                         func,
                         method /* = 'GET' */,
                         type /* = 'xml' */,
                         message /* = null */,
                         errfunc /* = null */)
    {
        if (typeof(method) == 'undefined')
            method = 'GET';
        if (typeof(type) == 'undefined')
            type = 'xml';
        if (typeof(message) == 'undefined')
            message = null;
            
        var self = this;
        this.bindObj.onreadystatechange = function () {
            if (self.bindObj.readyState == 4)
            {
                if (self.bindObj.status / 100 == 2) // Success
                    func(self.bindObj.responseText);
                else if (typeof(errfunc) == 'function')
                    errfunc(self.bindObj.responseText);
            }
        };
        
        this.bindObj.open(method,url,true);
        
    
        this.bindObj.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        this.bindObj.setRequestHeader("Connection", "close");
        
        
        if (type == 'formdata') {
            // Special case: objects treated as key/value pairs
            if (typeof(message) == 'object') {
                var messageObj = message;
                message = '';
                for (key in messageObj) {
                    message += (message != '' ? '&' : '') + URLEncode(key) + '=' + URLEncode(messageObj[key]);
                }
            }
            this.bindObj.setRequestHeader("Content-length", message.length);
        }
        this.bindObj.send(message);
    }
}