
/*
 * Session:
 * - id: the session identifier.
 * - clientId: the ID of the client we represent.
 * - name: the name of the session.
 * - active: whether the session is still active.
 * - masterClient: the ID of the master client.
 * - clients: an array of {'id': ..., 'name': ..., 'timeSinceLastUpdate': ...} client records.
 * - refresh(): refreshes the session's properties and calls a callback.
 * 
 * All fields except for id, clientId and the functions are initially set to null.
 */
function Session(clientId) {
    this.id = null;
    this.clientId = clientId;
    this.name = null;
    this.active = null;
    this.masterClient = null;
    this.clients = null;
    this.playerId = null;
    this.map = null;
    
    var self = this;
    
    this.refresh = function(callback, errorCallback /* = null */) {
        
        function success(data) {
            data = JSON.parse(data);
            
            self.id = data.session.id;
            self.map = data.session.map;
            self.name = data.session.name;
            self.active = data.session.joinable;
            self.masterClient = data.session.master_client;
            self.clients = data.players;
            self.chatMessages = data.session.chat_messages;
			self.playerId = data.players[self.clientId].playerid;
            
            if (typeof(callback) == 'function')
                callback(self);
        }
        
        var ajax = new Ajax();
        var request = {
			'c': this.clientId,
            'RT': 'SESSPROP'
        };
        ajax.bind('/webdune/server/gameserver.php', success, 'POST', 'formdata', request, errorCallback);
    }
	
	this.chatMessage = function(msg)
	{
		self.chatMessages = msg+self.chatMessages;
	}
	this.parsePlayerPings = function(serializedPings)
	{
		if (serializedPings)
			self.clients = JSON.parse(serializedPings);
	}
}
